#define std_io_InputTube_IMPORT

#include <stdlib.h>
#include <std/io/impl/InputTube.h>

void
InputTube_initialize(union InputTube in,
		     const struct InputTube_Methods *methods)
{
    in.impl->methods = methods;
    in.impl->hasError = 0;
}

ssize_t
InputTube_read(union InputTube in, void *data, size_t size)
{
    ssize_t s;

    if ((s = in.impl->methods->read(in.impl, data, size)) < 0) {
	in.impl->hasError = 1;
	return -1;
    }
    return s;
}

int
InputTube_readByte(union InputTube in)
{
    unsigned char byte;
    ssize_t s;

    if ((s = in.impl->methods->read(in.impl, &byte, sizeof(byte))) < 0) {
	in.impl->hasError = 1;
	return -1;
    }
    return (s == 0) ? InputTube_EOS : byte;
}

ssize_t
InputTube_readBytes(union InputTube in, void *top, size_t size)
{
    unsigned char *data = (unsigned char *)top;
    ssize_t delta;

    while (size > 0 && (delta = InputTube_read(in, data, size)) > 0) {
        data += delta;
        size -= delta;
    }
    return (size > 0 && delta < 0) ? -1 : data - (unsigned char *)top;
}

int
InputTube_readFully(union InputTube in, void *top, size_t size)
{
    unsigned char *data = (unsigned char *)top;
    ssize_t delta;

    while (size > 0 && (delta = InputTube_read(in, data, size)) > 0) {
        data += delta;
        size -= delta;
    }
    return (size > 0) ? -1 : 0;
}

int
InputTube_hasError(union InputTube in)
{
    return in.impl->hasError;
}

int
InputTube_close(union InputTube in)
{
    return in.impl->methods->close(in.impl);
}
